/* pc_mag_igrf -- IGRF elements summed up to n=m=10   040320
	coordinate system:
		geodetic WGS-84 (lat,lon) and altitude=0 km
		(for specific atlitude or geocentric system use program "igrf")
	date:
		given by decimal year (y-m-d is NOT accepted)
		ex.: 2020.2514 for 1 April 2020
		(for input of y-m-d date use program "igrf")
	Gauss coefficients:
		IGRF-13 (IAGA, Division V, Working Group V-MOD).
			2020: IGRF, 1945-2015: DGRF, 1900-1940: IGRF
		linear interpolation for between epocks
		linear extrapolations for >=2020 with SV values
	Notes:
		Sum of terms n<=10 adopted in this program or other "sunpmag" and
	"igrf" should be accurate enough for applications in paleomagnetism.
	For elements summed up to n=m=13, use the program at the following web sites,
	although it is the same n<=10 for years before 2000.
		https://www.ngdc.noaa.gov/IAGA/vmod/igrf.html
		http://wdc.kugi.kyoto-u.ac.jp/igrf/index-j.html
		http://geomag.bgs.ac.uk/data_service/models_compass/igrf_calc.html
*/

#include <math.h>
#include <stdlib.h>
#include "pcalc.h"

#define HEAD0 "geomagnetic elements of IGRF-13 up to n=m=10.\n\
  ** Geodetic coordinate system WGS-84 (lat,lon) & altitude = 0 km. **\n\
  ** Give the date in decimal year (ex.: 2020.3333 for 1 May 2020)  **\n\
  ** For more options such as geocentric system, use program \"igrf\" **\n\
Continue? (Rtn or Y/N)"
#define CONT " Slat    Slon  F(nT)   I      D\n%6.2f %7.2f %5.0f %6.2f %6.2f\n"
#define CONTP " Slat    Slon  F(nT)   I      D\n%6.2f %7.2f %5.0f %6.2f   *\n\
(At the poles D is indefinite: H=%5.0f)\n"
#define CONT2 "Year: %4.2f\n Slat    Slon  F(nT)   I      D\n"
#define CONT2a "%6.2f %7.2f %5.0f %6.2f %6.2f\n"
#define CONT2Pa "%6.2f %7.2f %5.0f %6.2f   *\n\
(At the poles D is indefinite: H=%5.0f)\n"
#define GEODETIC 0
#define HEIGHT 0.
#define LTNPOLE 89.9999
#define LTSPOLE -89.9999

extern FILE *Fp;
extern double Slt, Sln;
static int N=10, c;
static double year, **G, **H, Xa, Ya, Za, Ma, Ia, Da;

void pc_mag_igrf()
{
	fprintf(stderr, "%s", HEAD0);
	if (yorn("", +1) == ZERO) return;

	G = mallocmd(N+1, N+1);   H = mallocmd(N+1, N+1);
	while ((c=inpd("Decimal year?",1,&year)) != ZERO){
		if (c == -1) continue;
		if ((gausscoeff(year,N,G,H)) == -1){
			printf("** Span of year, 1900<=Year<=2025! **\n");
			continue;
		}
		printf("Year: %4.2f\n", year);
		while ((c=inpd("Slat,Slon?",2,&Slt,&Sln)) != ZERO){
			if (c == -1) continue;
			if (Slt<-90. || Slt>90. || Sln<-360. || Sln>360.){
				printf("** Error of latitude or longitude! **\n");
				continue;
			}
			gaussfld(GEODETIC, Slt, Sln, HEIGHT, N, &Xa, &Ya, &Za, G, H);
			carmag(Xa, Ya, Za, &Ma, &Ia, &Da);
			checklon(&Da, 1);
			if (Slt<LTNPOLE && Slt>LTSPOLE)
				printf(CONT, Slt, Sln, Ma, Ia, Da);
			else
				printf(CONTP, Slt, Sln, Ma, Ia, sqrt(Xa*Xa+Ya*Ya));
		}
	}
	freemd(N+1, G);   freemd(N+1, H);
}

/* pc_mag_igrf2 -- year, Slat, Slon are read from a file. */

void pc_mag_igrf2()
{
	char s1[30], s2[30], s3[30];
	int flagy = 0, flagy2 = 0;

	G = mallocmd(N+1, N+1);   H = mallocmd(N+1, N+1);
	year = 2020.;
	while ((c=get_s(Fp,3,s1,s2,s3)) != EOF){
		if (c==0 || s1[0]=='#' || s1[0]=='%') continue;
		if (s1[0] == '$'){
			flagy = 1;
			year = atof(s2);
			if ((gausscoeff(year,N,G,H)) == -1){
				fprintf(stderr, "%4.2f: ", year);
				error("Wrong year! (1900<=yr<=2025)");
			}
			printf(CONT2, year);
			continue;
		}
		if ((flagy==0) && (flagy2==0)){
			if ((gausscoeff(year,N,G,H)) == -1)
				error("Error of year!");
			printf(CONT2, year);
			flagy2 = 1;
		}
		Slt = atof(s1);   Sln = atof(s2);
		gaussfld(GEODETIC, Slt, Sln, HEIGHT, N, &Xa, &Ya, &Za, G, H);
		carmag(Xa, Ya, Za, &Ma, &Ia, &Da);
		checklon(&Da, 1);
		if (Slt<LTNPOLE && Slt>LTSPOLE)
			printf(CONT2a, Slt, Sln, Ma, Ia, Da);
		else
			printf(CONT2Pa, Slt, Sln, Ma, Ia, sqrt(Xa*Xa+Ya*Ya));

	}
	freemd(N+1, G);   freemd(N+1, H);
}
#undef HEAD0
#undef CONT
#undef CONTP
#undef CONT2
#undef CONT2a
#undef CONT2Pa
#undef GEODETIC
#undef HEIGHT
