#include <math.h>
#include "std.h"

#define NMIN	3
#define NBMIN	3

/* histn -- count data within each bin which is divided so as to be   0799,112918
	equal fraction of normal distribution with mean & stdv (which should be 
	supplied from the main, and can be independent from the data)
	Notes:
	    n, x -- number of data stored in array x (x[0]-x[n-1])
	    mean, stdev -- used to make devision of bins (not necessarily those of x)
	    xb[], zb[] -- boundaries of bins for original & normalized data
	    nbin -- number of bins (bins[0]-bins[nbin-1], etc.)
	    bin, ebin -- stores counts of data and theoritical expectation as,
		-oo < x < xb[0], xb[i] <= x < xb[i+1], xb[nbin-2] <= x < +oo
*/

void histn(n, x, mean, stdev, nbin, bin, ebin)
int n, nbin;
double *x, mean, stdev, *bin, *ebin;
{
	int j, k;
	double *zb, *xb, p0;

	if (n < NMIN) error("Error of data number in histn!");
	if (nbin < NBMIN) error("Error of bin number in histn!");
	zb = mallocvd(nbin-1);   xb = mallocvd(nbin-1);
	p0 = 1./nbin;
	for (k=0; k<nbin; k++) bin[k] = 0.;
	for (k=0; k<nbin; k++) ebin[k] = p0*n;
	for (j=0; j<nbin-1; j++){
		zb[j] = normtab(p0*(j + 1));
		xb[j] = zb[j]*stdev + mean;
	}
	countbin(n, x, nbin, xb, bin);
	freevd(zb);   freevd(xb);
}

/* histln  -- Given POSITIVE data x[i], x is transformed to y=log(x), i.e. x=e^y.
	Data number of each bin is counted with equal fraction for the histogram of
	NORMAL distribution of y, i.e. log-normal distribution of x. Mean my and
	standard deviation sy of y=log(x) (NOT x) should be supplied. The my and sy
	are not necessarily determined from the data, but using meanvarl() is easier.
	Variables:
	n, x -- number of data which is stored in array x (x[0]-x[n-1])
	my, sy -- mean & std dev of y=ln(x) which should be given from parent
	nbin -- number of bins (bins[0]-bins[nbin-1], etc.)
	bin, ebin -- stores counts of data & theoritical expectation
	zb[] -- boundaries of normalized y; z=(y-my)/sy=(log(x)-my)/sy
*/

void histln(n, x, my, sy, nbin, bin, ebin)
int n, nbin;
double *x, my, sy, *bin, *ebin;
{
	int i, j, k;
	double *y, *zb, *yb, p0, s = 0., s2 = 0.;

	if (n < NMIN) error("Error of data number in histln!");
	if (nbin < NBMIN) error("Error of bin number in histln!");
	y = mallocvd(n);
	for (i=0; i<n; i++){
		if (x[i] <= 0.) error("Zero/negative data in histln!");
		y[i] = log(x[i]);
	}
	zb = mallocvd(nbin-1);   yb = mallocvd(nbin-1);
	p0 = 1./nbin;
	for (k=0; k<nbin; k++) bin[k] = 0.;
	for (k=0; k<nbin; k++) ebin[k] = p0*n;
	for (j=0; j<nbin-1; j++){
		zb[j] = normtab(p0*(j + 1));
		yb[j] = zb[j]*sy + my;
	}
	countbin(n, y, nbin, yb, bin);
	freevd(y);  freevd(zb);  freevd(yb);
}

/* histf -- count data of angular distance (t) and declination (d) within   0799,1218
	each bin. Bins are divided in equal fraction of Fisher distribution with
	precesion parameter k for t, and homogeneous distribution for d. k is
	supplied from the parent function and can be independent from t.
	    t -- array of theta (co-inclination) in degrees (t[0]-t[n-1])
	    d -- array of declination around the center (d[0]-d[n-1])
	[NOTE]: t & d should be angular distance and declinaion, respectively,
	measured around a center, all of which should be supplied from the parent
	function. Calculation of (t,d) measured around the center (incc,decc)
	from the original data (inc,dec) can be processed as the followings,
	in which the center is arbitrary (usually, the mean dirction).
		rotuvec(90.-decc, 90.-incc, inc, dec, &dum1, &dum2);
		rotuvec(-90., 0., dum1, dum2, &inct, &dect);
		t = 90.-inct;  d = dect;
	(see, "fldadf" & "fldadp in fldvgp")
*/

void histf(n, t, d, kappa, nbin, bint, bind, ebint, ebind)
int n, nbin;
double *t, *d, kappa, *bint, *bind, *ebint, *ebind;
{
	int j, k;
	double *tb, *db, p0;

	if (n < NMIN) error("Error of data number in histf!");
	if (nbin < NBMIN) error("Error of bin number in histf!");
	tb = mallocvd(nbin-1);   db = mallocvd(nbin-1);
	p0 = 1./nbin;
	for (k=0; k<nbin; k++){ bint[k] = 0.;   bind[k] = 0.; }
	for (k=0; k<nbin; k++){ ebint[k] = p0*n;   ebind[k] = p0*n; }
	for (j=0; j<nbin-1; j++){
		tb[j] = fishtab(kappa, p0*(j+1));
		db[j] = 360.*p0*(j+1);
	}
	countbin(n, t, nbin, tb, bint);
	countbin(n, d, nbin, db, bind);
	freevd(tb);   freevd(db);
}

/* countbin -- for data x[i] (i=0...n-1) and bin boundary xb[j] (j=0...nbin-2),
	count data numbers in bin[j] (j=0...nbin-1). Datum on the boundary xb[j]
	is included as a floor datum to the bin[j+1].
	    bin[0]      ...       bin[j+1]       ...      bin[nbin-1]
	-oo < x < xb[0] ... xb[j] <= x < xb[j+1] ... xb[nbin-2] <= x < +oo
*/

void countbin(n, x, nb, xb, bin)
int n, nb;
double *x, *xb, *bin;
{
	int i, j, k;
	double *xs;

	xs = mallocvd(n);
	for (i=0; i<n; i++) xs[i] = x[i];
	sort(n, xs);
	j = k = 0;
	for (i=0; i<n; i++){
		if (xs[i] < xb[j])
			bin[k] += 1.;
		else {
			while (xs[i] >= xb[j]){
				if (j == nb-2){
					k = nb-1;
					break;
				}
				j++;  k++;
			}
			bin[k] += 1.;
		}
	}
	freevd(xs);
}
#undef NMIN
#undef NBMIN
/*
printf("p : "); for (j=0; j<nbin-1; j++) printf("%6.4f ",p0*(j+1)); printf("\n");
printf("zb: "); for (j=0; j<nbin-1; j++) printf("%6.4f ",zb[j]); printf("\n");
printf("xb: "); for (j=0; j<nbin-1; j++) printf("%6.4f ",xb[j]); printf("\n");

for (j=0; j<nbin-1; j++) printf("%4.1f ",tb[j]); printf("\n");
for (j=0; j<nbin-1; j++) printf("%4.1f ",db[j]); printf("\n");

printf("p : "); for (j=0; j<nbin-1; j++) printf("%6.4f ",p0*(j+1)); printf("\n");
printf("zb: "); for (j=0; j<nbin-1; j++) printf("%6.4f ",zb[j]); printf("\n");
printf("xb: "); for (j=0; j<nbin-1; j++) printf("%6.4f ",exp(zb[j]*sy+my)); printf("\n");
*/
