/* ludcmp -- LU decomposition of a matrix a(n x n).   083013
	amended from Press et al. 1992, Numerical Recepies in C, p.46
	In the main program, allocate the memory of matrix a and vector d as,
		a[n+1][n+1] and d[n+1]
	but use them only a[1][1] to a[n][n] and d[1] to d[n].
	Example of main program (see t_ludcmp):

		int *indx;
		double **a, d, *b;
		indx = mallocvi(5);
		a = mallocmd(5, 5);
		b = mallocvd(5);
		...
		ludcmp(a, 4, indx, &d);
		lubksb(a, 4, indx, b);
		...
		freevi(indx);
		freemd(5, a);
		freevd(b);
	Returns:
		-1 when singular matrix occurs, +1 otherwise.
*/

#include <math.h>
#include "std.h"

#define TINY 1.0e-20

int ludcmp(a, n, indx, d)
int n, *indx;
double **a, *d;
{
	int i, imax, j, k;
	double big, dum, sum, temp;
	double *vv;

	vv = mallocvd(n+1);
	*d = 1.0;
	for (i=1; i<=n; i++){
		big = 0.0;
		for (j=1; j<=n; j++)
			if ((temp = fabs(a[i][j])) > big) big = temp;
/*		if (big == 0.0) error("Error of singular matrix in ludcmp"); */
		if (big == 0.0){
			fprintf(stderr, "Error of singular matrix in ludcmp!\n");
			freevd(vv);
			return(-1);
		}
		vv[i] = 1.0/big;
	}
	for (j=1; j<=n; j++){
		for (i=1; i<j; i++){
			sum = a[i][j];
			for (k=1; k<i; k++) sum -= a[i][k]*a[k][j];
			a[i][j] = sum;
		}
		big = 0.0;
		for (i=j; i<=n; i++){
			sum = a[i][j];
			for (k=1; k<j; k++)
				sum -= a[i][k]*a[k][j];
			a[i][j] = sum;
			if ((dum = vv[i]*fabs(sum)) >= big){
				big = dum;
				imax = i;
			}
		}
		if (j != imax){
			for (k=1; k<=n; k++){
				dum = a[imax][k];
				a[imax][k] = a[j][k];
				a[j][k] = dum;
			}
			*d = -(*d);
			vv[imax] = vv[j];
		}
		indx[j] = imax;
		if (a[j][j] == 0.0) a[j][j] = TINY;
		if (j != n) {
			dum = 1.0/(a[j][j]);
			for (i=j+1; i<=n; i++) a[i][j] *= dum;
		}
	}
	freevd(vv);
	return(+1);
}


/* lubksb -- forward & backward substitution in A*X = B   083013
	amended from Press et al. 1992, Numerical Recepies in C, p.47
*/

void lubksb(a, n, indx, b)
int n, *indx;
double **a, *b;
{
	int i, ii = 0, ip, j;
	double sum;

	for (i=1; i<=n; i++){
		ip = indx[i];
		sum = b[ip];
		b[ip] = b[i];
		if (ii)
			for (j=ii; j<=i-1; j++) sum -= a[i][j]*b[j];
		else if (sum) ii = i;
		b[i] = sum;
	}
	for (i=n; i>=1; i--){
		sum = b[i];
		for (j=i+1; j<=n; j++) sum -= a[i][j]*b[j];
		b[i] = sum/a[i][i];
	}
}

#undef TINY
/* (C) Copr. 1986-92 Numerical Recipes Software SD&5-X=!+!. */
