/* mleb -- maximization of Bingham's m.l.e. function   0919,0597,0896 
	actually MINIMIZES -(m.l.e.) by powell
	static variables shared among functions are n, k1, k2, t1, t2.
	3-d vector k, 3x3 matrix vd are for use by powell
	mallocmd should be avoided to memory alloc **vd
Return:
	ZERO when error, otherwise +1.
Iteration stop:
	As it seems difficult to stop the process from the host,
	an easy temporary method was taken by static integer itm which
	counts the number of mledk() called, and stops when reached ITMMAX.
	ITMMAX -- now 200 is adopted so that difficult data such as MI35 or
		random data of N=3 reach convergence in Bingham statistics.
*/

#include <stdlib.h>
#include <math.h>
#include "std.h"

#define ITMMAX 200
#define CSTOP "** Sorry, MLE is diverging! Stopped at %d iterations. **\n"

static int n, itm;
static double k1, k2, t1, t2;

int mleb(nd, tmin, tintm, kmin, kintm)
int nd;
double tmin, tintm, *kmin, *kintm;
{
	int it;
	double k[3], **vd, eps = 1.e-8, fmin, mledk();

	itm = 1;
	n = nd;
	t1 = tmin;   t2 = tintm;
	if ( ( vd = (double **) malloc( (size_t)(3*sizeof(double*)) ) )
		== NULL ) error("error in memory allocation of vd");
	if ( ( *vd = (double *) malloc( (size_t)(9*sizeof(double)) ) )
		== NULL ) error("error in memory allocation of *vd");
	k[1] = 0.;   k[2] = 0.;			/* initial values of k1, k2 */
	*(*vd+4) = -1.;   *(*vd+7) = 0.;	/* initial search direction 1 */
	*(*vd+5) = 0.;   *(*vd+8) = -1.;	/* initial search direction 2 */
	if (powell(k, vd, 2, eps, &it, &fmin, mledk) == ZERO) return(ZERO);
	*kmin = k[1];   *kintm = k[2];
	free(vd);
	return(1);
}

/* mledk -- returns -m.l.e. of Bingham distribution 
	arguments are k1, k2 in the form of 3-d vector for use in powell()
	integration of dk() is made by quad2ds()
	Stop the process when itm reaches to ITMMAX   0919
*/

double mledk(pk)
double *pk;
{
	double dks, lg4pi = 2.531024247, dk();

	k1 = pk[1];   k2 = pk[2];
	dks = quad2ds(dk, 0., HPAI, 0., PAI)/PAI;	/* dk/4pi */
	fprintf(stderr, "** Wait %d **\r", itm++);
	fflush(stderr);
	if (itm >= ITMMAX){
		printf(CSTOP, itm);
		exit(1);
	}
	return n*(lg4pi + log(dks)) - k1*t1 - k2*t2;	/* actually, -m.l.e. */
}

/* dk -- function to be surface integrated to give the Bingham's normalization factor 4*pi*dk */

double dk(th, ph)
double th, ph;
{
	double sinth, cosph, sinph;

	sinth = sin(th);
	cosph = cos(ph);   sinph = sin(ph);
	return exp((k1*cosph*cosph + k2*sinph*sinph)*sinth*sinth)*sinth;
}

/* mlebmap -- create a file of table (k1,k2,mle) for use by GMT to draw a map of m.l.e.
	Example of later process by GMT:
	xyz2grd file.tbl -Gfile.grd -I1 -R-50/0/-50/0
	psbasemap -JX4/4 -R-50/0/-50/0 -Ba5f1:"k1":/a5f1:"k2"::."title":wEsN -P -X2 -Y3 -K >psfile
	grdcontour file.grd -JX4/4 -R-50/0/-50/0 -C0.2 -A2 -Wc1ta -Wa3 -P -O >>psfile
*/

void mlebmap(file, nd, tmin, tintm, kmin, kintm)
char *file;
int nd;
double tmin, tintm, kmin, kintm;
{
	extern unsigned Optflags;
	FILE *fp;
	double kk, klim, kintv, k[3], mledk();

	n = nd;
	t1 = tmin;   t2 = tintm;
	if ( (fp = fopen(file, "w")) == NULL )
		error("error of fopen in mlebmap.\n");
	kk = 2.*kmin;
	if (kk > -0.5) {klim = -0.5;   kintv = -0.01;}
	else if (kk > -1.) {klim = -1.;   kintv = -0.02;}
	else if (kk > -5.) {klim = -5.;   kintv = -0.1;}
	else if (kk > -10.) {klim = -10.;   kintv = -0.2;}
	else if (kk > -50.) {klim = -50.;   kintv = -1.;}
	else if (kk > -100.) {klim = -100.;   kintv = -2.;}
	else if (kk > -500.) {klim = -500.;   kintv = -10.;}
	else {klim = -1000.;   kintv = -20.;}
	for (k[1] = 0.; k[1] >= klim; k[1] += kintv){
		for (k[2] = 0.; k[2] >= klim; k[2] += kintv){
			fprintf(fp, "%5.2f %5.2f %9.4f\n", k[1], k[2], -mledk(k));
		}
	}
	fclose(fp);
}
#undef ITMMAX
/*
fprintf(stderr, "k1,k2,mle,it: %f  %f  %f  %d\n", k[1], k[2], -fmin, it);
fprintf(stderr, "vd1: %f %f\nvd2: %f %f\n", *(*vd+4), *(*vd+7), *(*vd+5), *(*vd+8)); 
	if (Optflags & VRFY_BIT)
		fprintf(stderr, "klim=%8.2f, kintv=%6.2f\n", klim, kintv);
	if (Optflags & VRFY_BIT)
		fprintf(stderr, "\n");
		if (Optflags & VRFY_BIT) 
			fprintf(stderr, "now doing k1=%8.2f\r", k[1]);
	klim = -pow(10., ceil(log10(fabs(kmin))));
	kintv = klim/100.;
*/
