#include <math.h>
#include <stdlib.h>
#include "std.h"

/* pcalin -- principal component analysis of a linear data
	V.1.1: direction of u conform with remanence direction x[0]-x[nd-1]   082603
	V.1.0: source code is after statb()   040802
    Arguments:
	x, y, z -- nd sets of (x, y, z).  Ordinary nd-dimension array, 
		and use as (0,1,2,...,nd-1)
	anch -- segment is anchored to origin (+1) or free (0)
	xb, yb, zb -- mean of the data, or center of mass
	ux, uy, uz -- eigen vector giving the linear segment (largest eigen value)
	mad -- maximum angular deviation around the segment (2nd eq. of Eq.3 in Kirschvink)
    Variables:
	T,V: T matrix and a matrix of eigen vectors in coluyzs.
	     4x4 matrixes, but use as (1,2,3)x(1,2,3)
	d: vector of eigen values.
	   4-dimension vector, but use as (1,2,3)
    Returns:
	ZERO when error in jacobi(), otherwise returns +1.
*/

int pcalin(x, y, z, nd, anch, xb, yb, zb, ux, uy, uz, mad)
int nd, anch;
double *x, *y, *z, *xb, *yb, *zb, *ux, *uy, *uz, *mad;
{
	int i, nrot, i1, i2, i3;
	double xc=0., yc=0., zc=0., xx=0., xy=0., xz=0., yy=0., yz=0., zz=0., \
		xic, yic, zic, **T, d[4], **V;

	if (nd < 1) error("pcalin: error of nd < 1!");
	if (nd == 1){
		if (anch != 1) error("pcalin: nd = 1 for non-anchored!");
		*xb = *ux = x[0];   *yb = *uy = y[0];   *zb = *uz = z[0];
		*mad = -1.;
		return(1);
	}
        if ( (T = (double **) malloc( (size_t)(4*sizeof(double*)) ) )
                == NULL ) error("error in memory allocation of T");
        if ( (*T = (double *) malloc( (size_t)(16*sizeof(double)) ) )
                == NULL ) error("error in memory allocation of *T");
        if ( (V = (double **) malloc( (size_t)(4*sizeof(double*)) ) )
                == NULL ) error("error in memory allocation of V");
        if ( (*V = (double *) malloc( (size_t)(16*sizeof(double)) ) )
                == NULL ) error("error in memory allocation of *V");
/*
	T = mallocmd(4, 4);   V = mallocmd(4, 4);
*/
	if (anch == 0){
		for (i = 0; i < nd; i++){
			xc += x[i];   yc += y[i];   zc += z[i];
		}
		*xb = xc /= nd;   *yb = yc /= nd;   *zb = zc /= nd;
	}
	else if (anch == 1){
		*xb = xc = 0.;   *yb = yc = 0.;   *zb = zc = 0.;
	}
	else
		error("Wrong anch in pcalin!");
	for (i = 0; i < nd; i++){
		xic = x[i] - xc;   yic = y[i] - yc;   zic = z[i] - zc;
		xx += xic*xic;   xy += xic*yic;   xz += xic*zic;
		yy += yic*yic;   yz += yic*zic;   zz += zic*zic;
	}
	*(*T+5) = xx;   *(*T+6) = xy;   *(*T+7) = xz;
	*(*T+9) = xy;   *(*T+10) = yy;   *(*T+11) = yz;
	*(*T+13) = xz;   *(*T+14) = yz;   *(*T+15) = zz;
	if (jacobi(T, 3, d, V, &nrot) == ZERO){
/*		freemd(4, T);   freemd(4, V); */
		free(T);   free(V);
		return(ZERO);
	}
	eigenorder(d, &i1, &i2, &i3);
	*ux = *(*V+4+i3);   *uy = *(*V+8+i3);   *uz = *(*V+12+i3);
	check_usign(x, y, z, nd, ux, uy, uz);
	if (nd > 2)
		*mad = RAD*atan( sqrt( (d[i1] + d[i2])/d[i3] ) );
	else
		*mad = 0.;
/*	freemd(4, T);   freemd(4, V); */
	free(T);   free(V);
	return(1);
}

/* check_usign -- force the largest eigen vector u to the same direction as x[0]-x[n-1] */

void check_usign(x, y, z, n, ux, uy, uz)
int n;
double *x, *y, *z, *ux, *uy, *uz;
{
	double dx, dy, dz;

	dx = x[0] - x[n-1];   dy = y[0] - y[n-1];   dz = z[0] - z[n-1];
	if ( (fabs(dx) >= fabs(dy)) && (fabs(dx) >= fabs(dz)) ){
		if (dx*(*ux) < 0.){
			*ux = -(*ux);   *uy = -(*uy);   *uz = -(*uz);
		}
		return;
	}
	if ( (fabs(dy) >= fabs(dz)) && (fabs(dy) >= fabs(dx)) ){
		if (dy*(*uy) < 0.){
			*ux = -(*ux);   *uy = -(*uy);   *uz = -(*uz);
		}
		return;
	}
	if ( (fabs(dz) >= fabs(dx)) && (fabs(dz) >= fabs(dy)) ){
		if (dz*(*uz) < 0.){
			*ux = -(*ux);   *uy = -(*uy);   *uz = -(*uz);
		}
		return;
	}
	error("check_usign: something wrong in cheking the signe of u-vector!");
}
/*
memory of 4x4 matrix which is used to calculate 3x3 matrix
	 0   1   2   3
	 4   5   6   7
	 8   9  10  11
	12  13  14  15
*/

/* pcalina -- automatic PCA of a line with algorithm simplified from Kirshvink (1980),     060518
		which seeks only one component of the highest Tb or Hc. PCA is done by pcalin.
    Arguments:
	madc -- critical MAD to be a linear line
	I1, I2 -- selected range of steps to cover the linear line
    Returns:
	ZERO when no linear line was found, otherwise returns +1.
    For other details, see the notes in pcalin.
*/

int pcalina(x, y, z, nd, madc, origin, anch, xb, yb, zb, ux, uy, uz, I1, I2, mad)
int nd, origin, anch, *I1, *I2;
double *x, *y, *z, madc, *xb, *yb, *zb, *ux, *uy, *uz, *mad;
{
	int i, i1, i2, ni;
/*	double xi[nd+1], yi[nd+1], zi[nd+1], madi = 0.; */
	double *xi, *yi, *zi, madi = 0.;

	xi = mallocvd(nd+1);  yi = mallocvd(nd+1);  zi = mallocvd(nd+1);
	if (origin == 1){
		if (nd < 2) error("pcalina: error of nd < 2!");
		xi[nd] = yi[nd] = zi[nd] = 0.;
		i1 = nd -2;  i2 = nd;
	}
	else {
		if (nd < 3) error("pcalina: error of nd < 3!");
		i1 = nd -3;  i2 = nd - 1;
	}
	for (i=0; i<=nd-1; i++){
		xi[i] = x[i];  yi[i] = y[i];  zi[i] = z[i];
	}
	ni = i2 - i1 + 1;
	pcalin(xi+i1, yi+i1, zi+i1, ni, anch, xb, yb, zb, ux, uy, uz, &madi);
/*	printf("Check-0:i1,i2,ni,madi: %d %d %d %.1f\n",i1,i2,ni,madi); */

/* shift a window */

	while (madi > madc){
		if (i1 >= 1){
			i1 -= 1;  i2 -= 1;
			ni = i2 - i1 + 1;
			pcalin(xi+i1, yi+i1, zi+i1, ni, anch, xb, yb, zb, ux, uy, uz, &madi);
/*			printf("Check-1:i1,i2,ni,madi: %d %d %d %.1f\n",i1,i2,ni,madi); */
			continue;
		}
		else {
/*			printf("Check-2:i1,i2,ni,madi: %d %d %d %.1f\n",i1,i2,ni,madi); */
			*I1 = i1;  *I2 = i2;  *mad = madi;
			freevd(xi);  freevd(yi);  freevd(zi);
			return ZERO;  /* No linear three consecutive points */
		}
	}

/* expand i1 toward i1=0 */

	if (i1 >= 1){
		i1 -= 1;
		ni = i2 - i1 + 1;
		pcalin(xi+i1, yi+i1, zi+i1, ni, anch, xb, yb, zb, ux, uy, uz, &madi);
/*		printf("Check-3:i1,i2,ni,madi: %d %d %d %.1f\n",i1,i2,ni,madi); */
		while (madi <= madc){
			if (i1 >= 1){
				i1 -= 1;
				ni = i2 - i1 + 1;
				pcalin(xi+i1, yi+i1, zi+i1, ni, anch, xb, yb, zb, ux, uy, uz, &madi);
/*				printf("Check-4:i1,i2,ni,madi: %d %d %d %.1f\n",i1,i2,ni,madi); */
				continue;
			}
			else {
				i1 -= 1;  /* set i1=-1 so that later linear for i1=0 to i2 */
/*				printf("Check-5:i1,i2,ni,madi: %d %d %d %.1f\n",i1,i2,ni,madi); */
				break;
			}
		}
		i1 += 1;  /* Restore i1 */
/*		printf("Check-6:i1,i2,ni,madi: %d %d %d %.1f\n",i1,i2,ni,madi); */
	}

/* expand i2 toward i2=nd-1 */

	if (i2 <= nd-2){
		i2 += 1;
		ni = i2 - i1 + 1;
		pcalin(xi+i1, yi+i1, zi+i1, ni, anch, xb, yb, zb, ux, uy, uz, &madi);
/*		printf("Check-7:i1,i2,ni,madi: %d %d %d %.1f\n",i1,i2,ni,madi); */
		while (madi <= madc){
			if (i2 <= nd-2){
				i2 += 1;
				ni = i2 - i1 + 1;
				pcalin(xi+i1, yi+i1, zi+i1, ni, anch, xb, yb, zb, ux, uy, uz, &madi);
/*				printf("Check-8:i1,i2,ni,madi: %d %d %d %.1f\n",i1,i2,ni,madi); */
				continue;
			}
			else {
				i2 += 1;  /* set i2=nd so that later linear for i1 to i2=nd-1 */
/*				printf("Check-9:i1,i2,ni,madi: %d %d %d %.1f\n",i1,i2,ni,madi); */
				break;
			}
		}
		i2 -= 1;  /* Restore i2 */
/*		printf("Check-10:i1,i2,ni,madi: %d %d %d %.1f\n",i1,i2,ni,madi); */
	}

/* do pcalin with final i1 & i2, and return linear results */

	ni = i2 - i1 + 1;
	pcalin(xi+i1, yi+i1, zi+i1, ni, anch, xb, yb, zb, ux, uy, uz, &madi);
/*	printf("Check-11:i1,i2,ni,madi: %d %d %d %.1f\n",i1,i2,ni,madi); */
	*mad = madi;
	*I1 = i1;
	if (origin == 1){
		if (i2 == nd) *I2 = i2 - 1;
		else *I2 = i2;
	}
	else
		*I2 = i2;
	freevd(xi);  freevd(yi);  freevd(zi);
	return 1;
}
