/* sunpmag -- core azimuth from the sun orientation data V.2.3  040320,0319,1097,0989,0688
    This program is applicable ONLY to the data measured by the orientation device
	supplied by Natsuhara Giken. Calculation of the sun azimuth is based on
	the astronomical algorithm of Yallop and Hohenkerk (2007) in which the
	claimed error of the sun's azimuth is ~0.6 minutes for the time span of
	2000 BC to AD 2200 (NAO Tech. Note No.67, HM Nautical Almanac Office,
	U.K. Hydrographic Office).
    Input data:
	lat,lon -- decimal latitude & longitude of the site locality
	tzone -- east positive time difference (ex., +9 for Japan)
	dec -- optional local declination (when omitted, IGRF-13 value is given
		with n=m=10 for the geodetic WGS-84 system with altitude 0 km)
	plun -- plunge of the core (not used in the azimuth calculation)
	ref_azim -- optional reference azimuth, eastward positive from the true north
	mag -- angle of a marker measured COUNTERCLOCKWISE from the magnetic north
	sun -- angle of the sun measured clockwise from a marker (NOT sun's shadow)
	ref -- angle of a reference measured clockwise from a marker
    Input file format:
	$$ lat lon tzone [dec] -- line with "$$" at the top is the locality
	$ year month day [ref_azim]    -- site data begins after the line with "$"
	core1-1 plun mag               -- case of plun and mag only
	core1-2 plun mag sun h m s     -- case of plun, mag, sun, and time
	core1-3 plun mag ref           -- case of plun, mag, and ref
	core1-4 plun mag sun h m s ref -- case of all measurements
		.
		.
	$ year month day [ref_azim]    -- another site data begins
	core2-1 plun mag sun h m s
		.
	$$ lat lon tzone [dec]         -- another locality begins
		.
    Output File:
	eastward positive azimuth of the marker is print out to "stdout" together
	with the input data. To save them to a file use UNIX's re-direction as
	"sunpmag in_file > out_file"
*/

#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include "sunpmag.h"

char ID[100];
double plun, mazim, cmag;

int main(argc, argv)
int argc;
char *argv[];
{
	FILE *fp;
	char file[500], s1[50], s2[50], s3[50], s4[50], s5[50], s6[50], s7[50], s8[50], *s;
	int n, flagf=0, flags=0, flagd=0, iyr=0, iyrp=0, imo=0, imop=0, idy=0;
	double dyear, lat, lon, sld, dec, yr, mo, dy, hr, mi, sc, sazim, razim, \
		sunn, sundr, ref, refdr = 0., deltms, deltmr, deltrs, dum;

	if (argc < 2) error(USAGE);
	while (--argc > 0){
		if (*argv[argc] == '-'){
			s = argv[argc] + 1;
			if ((toupper(*s)) == 'H') showless(HELP);
			else error("Unknown option!");
			break;
		}
		else {
			if (flagf != 0) error("Assign only one file!");
			strcpy(file, argv[argc]);
			if ((fp=fopen(file, "r")) == NULL) error("Error of fopen!");
			flagf = 1;
		}
	}
	print1(file);
	while ((n=get_s(fp,8,s1,s2,s3,s4,s5,s6,s7,s8)) != EOF){
		if (n==0 || s1[0]=='#' || s1[0]=='%') continue;
		if (strcmp(s1,"$$") == 0){
			if (n < 4) error("sunpmag: error in $$ line!");
			lat = atof(s2);  lon = atof(s3);  sld = atof(s4);
			if (n >= 5){
				dec = atof(s5);
				print2(lat, lon, sld, dec);
				flagd = 0;
			}
			else if (n == 4){
				flagd = 1;
				flags = 1;
			}
		}
		else if (strcmp(s1,"$") == 0){
			if (n < 4) error("sunpmag: error in $ line!");
			yr = atof(s2);  mo = atof(s3);  dy = atof(s4);
			iyr = (int)yr;  imo = (int)mo;  idy = (int)dy;
			if (flags==1 && flagd==1){
				dec = igrfdec(iyr, imo, idy, lat, lon, &dyear);
				print2a(lat, lon, sld, dec, dyear);
				iyrp = iyr;   imop = imo;
				flags = 0;
			}
			else if (flags==0 && flagd==1){
				if (iyr!=iyrp || imo!=imop){
					dec = igrfdec(iyr, imo, idy, lat, lon, &dyear);
					print2a(lat, lon, sld, dec, dyear);
					iyrp = iyr;
				}
			}
			if (n == 4)
				print3(yr, mo, dy);
			else if (n == 5){
				refdr = atof(s5);
				print4(yr, mo, dy, refdr);
			}
		}
		else {
			if (n==1 || n==2) error("sunpmag: error in a data line!");
			strcpy(ID, s1);
			plun = atof(s2);  cmag = atof(s3);
			dum = 360. - cmag;
			mazim = dum + dec;
			if (mazim < 0.)   mazim += 360.;
			if (mazim > 360.) mazim -= 360.;
			if (n == 3)
				print5();
			else if (n == 4){
				ref = sunn = atof(s4);  razim = refdr - ref;
				if (razim < 0.)   razim += 360.;
                                if (razim > 360.) razim -= 360.;
				deltmr = mazim - razim;
				print6(razim, deltmr, ref);
			}
			else if (n==7 || n==8){
				sunn = atof(s4);
				hr = atof(s5);  mi = atof(s6);  sc = atof(s7);
				if (n == 8) ref = atof(s8);
				sundr = sundir(lat,lon,yr,mo,dy,hr,mi,sc,sld,&dum) + 180.;
				sazim = sundr - sunn;
				if (sazim < 0.) sazim += 360.;
				if (sazim > 360.) sazim -= 360.;
				deltms = mazim - sazim;
				if (deltms > 180.)   deltms -= 360.;
				if (n == 7)
				      print7(sazim, deltms, sunn, hr, mi, sc);
				else {
				      razim = refdr - ref;
                                      if (razim < 0.)   razim += 360.;
                                      if (razim > 360.) razim -= 360.;
                                      deltmr = mazim - razim;
				      deltrs = razim - sazim;
				      print8(sazim,deltms,razim,deltmr,deltrs,sunn,hr,mi,sc,ref);
				}
			}
		}
	}
        fclose(fp);
}

/* printout functions follow */

void print1(file)
char *file;
{
	fprintf(stdout, "Core Azimus Calculation of %s\n", file);
}

void print2(lat, lon, sld, dec)
double lat, lon, sld, dec;
{
	fprintf(stdout, "\n  Lat      Lon     Sld     Dec\n%7.3f  %7.3f %6.1f %6.1f\n", \
		lat, lon, sld, dec);
}

void print2a(lat, lon, sld, dec, dyear)
double lat, lon, sld, dec, dyear;
{
	fprintf(stdout, "\n  Lat      Lon     Sld  Dec(%8.3f)\n%7.3f  %7.3f %6.1f %6.1f\n", \
		dyear, lat, lon, sld, dec);
}

void print3(yr, mo, dy)
double yr, mo, dy;
{
	fprintf (stdout, "\n Year Mon Day Ref_azm\n%5.0f %3.0f %3.0f    *\n\n", yr, mo, dy);
	fprintf (stdout, " Core      Dip Sazim Mazim  dltMS Razim  ");
	fprintf (stdout, "dltMR  dltRS  Mag   Sun  Hr Mn Sc  Ref\n");
}

void print4(yr, mo, dy, refdr)
double yr, mo, dy, refdr;
{
	fprintf(stdout, "\n Year Mon Day Ref_azm\n%5.0f %3.0f %3.0f  %6.1f\n\n", \
		yr, mo, dy, refdr);
	fprintf(stdout, " Core      Dip Sazim Mazim  dltMS Razim  ");
        fprintf(stdout, "dltMR  dltRS  Mag   Sun  Hr Mn Sc  Ref\n");
}

/* print5 -- magnetic data only */

void print5()
{
	fprintf (stdout, "%-8s %5.1f   *   %5.1f    *     *      ", ID, plun, mazim);
	fprintf (stdout, "*      *   %5.1f   *    *  *  *   *\n", cmag);
}

/* print6 -- magnetic + reference measurements */

void print6(razim, deltmr, ref)
double razim, deltmr, ref;
{
	fprintf(stdout, "%-8s %5.1f   *   %5.1f    *   %5.1f ", ID, plun, mazim, razim);
	fprintf(stdout, "%6.1f    *   %5.1f   *    *  *  * %5.1f\n", deltmr, cmag, ref);
}

/* print7 -- magnetic + sun measurements */

void print7(sazim, deltms, sunn, hr, mi, sc)
double sazim, deltms, sunn, hr, mi, sc;
{
	fprintf(stdout, "%-8s %5.1f %5.1f %5.1f %6.1f", ID, plun, sazim, mazim, deltms);
	fprintf(stdout, "   *      *      *   %5.1f %5.1f %2.0f %2.0f %2.0f   *\n", \
		cmag, sunn, hr, mi, sc);
}

/* print8 -- all measurements (magnetic + sun + reference) */

void print8(sazim, deltms, razim, deltmr, deltrs, sunn, hr, mi, sc, ref)
double sazim, deltms, razim, deltmr, deltrs, sunn, hr, mi, sc, ref;
{
	fprintf(stdout, "%-8s %5.1f %5.1f %5.1f %6.1f ", ID,plun,sazim,mazim,deltms);
	fprintf(stdout, "%5.1f %6.1f %6.1f %5.1f %5.1f %2.0f %2.0f %2.0f %5.1f\n", \
		razim, deltmr, deltrs, cmag, sunn, hr, mi, sc, ref);
}
